// Copyright 2023 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Auto generated by tools/codegen/core/gen_experiments.py

#include "src/core/lib/experiments/experiments.h"

#include <grpc/support/port_platform.h>
#include <stdint.h>

#ifndef GRPC_EXPERIMENTS_ARE_FINAL

#if defined(GRPC_CFSTREAM)
namespace {
const char* const description_buffer_list_deletion_prep =
    "Gate the removal of old TCP timestamp collection mechanism.";
const char* const additional_constraints_buffer_list_deletion_prep = "{}";
const char* const description_call_tracer_in_transport =
    "Transport directly passes byte counts to CallTracer.";
const char* const additional_constraints_call_tracer_in_transport = "{}";
const char* const
    description_call_tracer_send_initial_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_initial_metadata_is_an_annotation =
        "{}";
const char* const
    description_call_tracer_send_trailing_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation =
        "{}";
const char* const description_callv3_batch_validation =
    "If enabled, perform validations on receiving a new batch.";
const char* const additional_constraints_callv3_batch_validation = "{}";
const char* const description_chaotic_good_framing_layer =
    "Enable the chaotic good framing layer.";
const char* const additional_constraints_chaotic_good_framing_layer = "{}";
const char* const description_chaotic_good_send_supported_features =
    "Send supported features in chaotic good settings frames.";
const char* const additional_constraints_chaotic_good_send_supported_features =
    "{}";
const char* const description_custom_frame_check =
    "Adding checks to custom HTTP2 frame.";
const char* const additional_constraints_custom_frame_check = "{}";
const char* const description_error_flatten =
    "Flatten errors to ordinary absl::Status form.";
const char* const additional_constraints_error_flatten = "{}";
const char* const description_event_engine_client =
    "Use EventEngine clients instead of iomgr's grpc_tcp_client";
const char* const additional_constraints_event_engine_client = "{}";
const char* const description_event_engine_dns =
    "If set, use EventEngine DNSResolver for client channel resolution";
const char* const additional_constraints_event_engine_dns = "{}";
const char* const description_event_engine_dns_non_client_channel =
    "If set, use EventEngine DNSResolver in other places besides client "
    "channel.";
const char* const additional_constraints_event_engine_dns_non_client_channel =
    "{}";
const char* const description_event_engine_fork =
    "Enables event engine fork handling, including onfork events and file "
    "descriptor generations";
const char* const additional_constraints_event_engine_fork = "{}";
const char* const description_event_engine_listener =
    "Use EventEngine listeners instead of iomgr's grpc_tcp_server";
const char* const additional_constraints_event_engine_listener = "{}";
const char* const description_event_engine_callback_cq =
    "Use EventEngine instead of the CallbackAlternativeCQ.";
const char* const additional_constraints_event_engine_callback_cq = "{}";
const uint8_t required_experiments_event_engine_callback_cq[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_for_all_other_endpoints =
    "Use EventEngine endpoints for all call sites, including direct uses of "
    "grpc_tcp_create.";
const char* const additional_constraints_event_engine_for_all_other_endpoints =
    "{}";
const uint8_t required_experiments_event_engine_for_all_other_endpoints[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineDns),
    static_cast<uint8_t>(
        grpc_core::kExperimentIdEventEngineDnsNonClientChannel),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_poller_for_python =
    "Enable event engine poller in gRPC Python";
const char* const additional_constraints_event_engine_poller_for_python = "{}";
const char* const description_fail_recv_metadata_on_deadline_exceeded =
    "Fail recv initial metadata when the deadline is exceeded.";
const char* const
    additional_constraints_fail_recv_metadata_on_deadline_exceeded = "{}";
const char* const description_free_large_allocator =
    "If set, return all free bytes from a \042big\042 allocator";
const char* const additional_constraints_free_large_allocator = "{}";
const char* const description_fuse_filters =
    "If set, individual filters are merged into fused filters";
const char* const additional_constraints_fuse_filters = "{}";
const char* const description_h2_max_deallocating_streams_headroom =
    "Separate allocated max concurrent streams.";
const char* const additional_constraints_h2_max_deallocating_streams_headroom =
    "{}";
const char* const description_inproc_cancel_stream =
    "If set, cancel inproc stream inside the transport mutex.";
const char* const additional_constraints_inproc_cancel_stream = "{}";
const char* const description_keep_alive_ping_timer_batch =
    "Avoid explicitly cancelling the keepalive timer. Instead adjust the "
    "callback to re-schedule itself to the next ping interval.";
const char* const additional_constraints_keep_alive_ping_timer_batch = "{}";
const char* const description_local_connector_secure =
    "Local security connector uses TSI_SECURITY_NONE for LOCAL_TCP "
    "connections.";
const char* const additional_constraints_local_connector_secure = "{}";
const char* const description_max_inflight_pings_strict_limit =
    "If set, the max inflight pings limit is strictly enforced.";
const char* const additional_constraints_max_inflight_pings_strict_limit = "{}";
const char* const description_memory_optimization_01 = "Memory Optimization";
const char* const additional_constraints_memory_optimization_01 = "{}";
const char* const description_memory_optimization_02 = "Memory Optimization 02";
const char* const additional_constraints_memory_optimization_02 = "{}";
const char* const description_message_size_refactoring =
    "Refactoring the message size config parsing.";
const char* const additional_constraints_message_size_refactoring = "{}";
const char* const description_metadata_outstanding_token_refactor =
    "Refactor ClientInitialMetadataOutstandingToken.";
const char* const additional_constraints_metadata_outstanding_token_refactor =
    "{}";
const char* const description_metadata_publish_to_app_tag =
    "Publish metadata to the app using the kPublishToApp metadata field.";
const char* const additional_constraints_metadata_publish_to_app_tag = "{}";
const char* const description_monitoring_experiment =
    "Placeholder experiment to prove/disprove our monitoring is working";
const char* const additional_constraints_monitoring_experiment = "{}";
const char* const description_multiping =
    "Allow more than one ping to be in flight at a time by default.";
const char* const additional_constraints_multiping = "{}";
const char* const description_optimization_01 = "Optimization";
const char* const additional_constraints_optimization_01 = "{}";
const char* const description_optimization_02 = "Optimization";
const char* const additional_constraints_optimization_02 = "{}";
const char* const description_optimization_03 = "Optimization";
const char* const additional_constraints_optimization_03 = "{}";
const char* const description_optimization_04 = "Optimization";
const char* const additional_constraints_optimization_04 = "{}";
const char* const description_otel_export_telemetry_domains =
    "Export telemetry domains in OpenTelemetry metrics.";
const char* const additional_constraints_otel_export_telemetry_domains = "{}";
const char* const description_ph2_client =
    "Use promises for the http2 client transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test PH2 Client "
    "with CHTTP2 Server.";
const char* const additional_constraints_ph2_client = "{}";
const char* const description_ph2_client_server =
    "This is a combination of ph2_server and ph2_client experiments. When this "
    "flag is enabled for the test suites, it would test PH2 Client with PH2 "
    "Server. We will NEVER be rolling out this experiment, because the roll "
    "out would be done using ph2_server and ph2_client. This experiment MUST "
    "be deleted when either ph2_server or ph2_client experiments are deleted.";
const char* const additional_constraints_ph2_client_server = "{}";
const char* const description_ph2_perf_01 =
    "Different performance experiments for PH2 Transport";
const char* const additional_constraints_ph2_perf_01 = "{}";
const char* const description_ph2_server =
    "Use promises for the http2 server transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test CHTTP2 "
    "Client with PH2 Server.";
const char* const additional_constraints_ph2_server = "{}";
const char* const description_pick_first_ignore_empty_updates =
    "Ignore empty resolutions in pick_first (delete)";
const char* const additional_constraints_pick_first_ignore_empty_updates = "{}";
const char* const description_pipelined_read_secure_endpoint =
    "Enable pipelined reads for EventEngine secure endpoints";
const char* const additional_constraints_pipelined_read_secure_endpoint = "{}";
const uint8_t required_experiments_pipelined_read_secure_endpoint[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_pollset_alternative =
    "Code outside iomgr that relies directly on pollsets will use non-pollset "
    "alternatives when enabled.";
const char* const additional_constraints_pollset_alternative = "{}";
const uint8_t required_experiments_pollset_alternative[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_prioritize_finished_requests =
    "Prioritize flushing out finished requests over other in-flight requests "
    "during transport writes.";
const char* const additional_constraints_prioritize_finished_requests = "{}";
const char* const description_promise_based_inproc_transport =
    "Use call-v3 for the in-process transport.";
const char* const additional_constraints_promise_based_inproc_transport = "{}";
const char* const description_promise_batch_cleanup_on_cancel =
    "Fix completion queue tag handling during promise batch cancellation.";
const char* const additional_constraints_promise_batch_cleanup_on_cancel = "{}";
const char* const description_promise_filter_send_cancel_metadata =
    "Enables sending all trailing metadata fields from server side "
    "promise-based filters upon stream cancellation.";
const char* const additional_constraints_promise_filter_send_cancel_metadata =
    "{}";
const char* const description_retry_in_callv3 = "Support retries with call-v3";
const char* const additional_constraints_retry_in_callv3 = "{}";
const char* const description_return_preexisting_errors =
    "Return errors that exist before the start of the call in RunHandler.";
const char* const additional_constraints_return_preexisting_errors = "{}";
const char* const description_schedule_cancellation_over_write =
    "Allow cancellation op to be scheduled over a write";
const char* const additional_constraints_schedule_cancellation_over_write =
    "{}";
const char* const description_secure_endpoint_offload_large_reads =
    "If a large read needs to be decrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_reads =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_reads[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_offload_large_writes =
    "If a large read write to be encrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_writes =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_writes[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_read_coalescing =
    "Coalesce decrypted bytes in SecureEndpoint if a read_hint is provided.";
const char* const additional_constraints_secure_endpoint_read_coalescing = "{}";
const uint8_t required_experiments_secure_endpoint_read_coalescing[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_skip_clear_peer_on_cancellation =
    "If set, skips clearing of peer string on call cancellation.";
const char* const additional_constraints_skip_clear_peer_on_cancellation = "{}";
const char* const description_subchannel_connection_scaling =
    "Subchannel connection scaling support.";
const char* const additional_constraints_subchannel_connection_scaling = "{}";
const char* const description_subchannel_wrapper_cleanup_on_orphan =
    "Fixes the subchannel wrapper to drop any non-cancelled watchers when it "
    "gets orphaned.";
const char* const additional_constraints_subchannel_wrapper_cleanup_on_orphan =
    "{}";
const char* const description_tcp_frame_size_tuning =
    "If set, enables TCP to use RPC size estimation made by higher layers. TCP "
    "would not indicate completion of a read operation until a specified "
    "number of bytes have been read over the socket. Buffers are also "
    "allocated according to estimated RPC sizes.";
const char* const additional_constraints_tcp_frame_size_tuning = "{}";
const char* const description_tcp_rcv_lowat =
    "Use SO_RCVLOWAT to avoid wakeups on the read path.";
const char* const additional_constraints_tcp_rcv_lowat = "{}";
const char* const description_tsi_frame_protector_without_locks =
    "Do not hold locks while using the tsi_frame_protector.";
const char* const additional_constraints_tsi_frame_protector_without_locks =
    "{}";
const char* const description_unconstrained_max_quota_buffer_size =
    "Discard the cap on the max free pool size for one memory allocator";
const char* const additional_constraints_unconstrained_max_quota_buffer_size =
    "{}";
const char* const description_use_call_event_engine_in_completion_queue =
    "Use the call event engine to run callbacks in completion queue.";
const char* const
    additional_constraints_use_call_event_engine_in_completion_queue = "{}";
const char* const description_v2_non_owning_waker_implementation =
    "Use non-owning wakers in v2 filter bridge.";
const char* const additional_constraints_v2_non_owning_waker_implementation =
    "{}";
const char* const description_verbose_channelz_connection_logging =
    "Verbose logging of events into channelz during connection setup.";
const char* const additional_constraints_verbose_channelz_connection_logging =
    "{}";
const char* const description_wildcard_ip_expansion_restriction =
    "If set, adds optional restriction on when to expand wildcard IPs.";
const char* const additional_constraints_wildcard_ip_expansion_restriction =
    "{}";
const char* const description_xds_server_filter_chain_per_route =
    "xDS servers use a separate filter chain for each route.";
const char* const additional_constraints_xds_server_filter_chain_per_route =
    "{}";
}  // namespace

namespace grpc_core {

const ExperimentMetadata g_experiment_metadata[] = {
    {"buffer_list_deletion_prep", description_buffer_list_deletion_prep,
     additional_constraints_buffer_list_deletion_prep, nullptr, 0, false, true},
    {"call_tracer_in_transport", description_call_tracer_in_transport,
     additional_constraints_call_tracer_in_transport, nullptr, 0, true, false},
    {"call_tracer_send_initial_metadata_is_an_annotation",
     description_call_tracer_send_initial_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_initial_metadata_is_an_annotation,
     nullptr, 0, true, true},
    {"call_tracer_send_trailing_metadata_is_an_annotation",
     description_call_tracer_send_trailing_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation,
     nullptr, 0, false, true},
    {"callv3_batch_validation", description_callv3_batch_validation,
     additional_constraints_callv3_batch_validation, nullptr, 0, true, true},
    {"chaotic_good_framing_layer", description_chaotic_good_framing_layer,
     additional_constraints_chaotic_good_framing_layer, nullptr, 0, true,
     false},
    {"chaotic_good_send_supported_features",
     description_chaotic_good_send_supported_features,
     additional_constraints_chaotic_good_send_supported_features, nullptr, 0,
     false, true},
    {"custom_frame_check", description_custom_frame_check,
     additional_constraints_custom_frame_check, nullptr, 0, true, false},
    {"error_flatten", description_error_flatten,
     additional_constraints_error_flatten, nullptr, 0, true, false},
    {"event_engine_client", description_event_engine_client,
     additional_constraints_event_engine_client, nullptr, 0, true, false},
    {"event_engine_dns", description_event_engine_dns,
     additional_constraints_event_engine_dns, nullptr, 0, true, false},
    {"event_engine_dns_non_client_channel",
     description_event_engine_dns_non_client_channel,
     additional_constraints_event_engine_dns_non_client_channel, nullptr, 0,
     true, false},
    {"event_engine_fork", description_event_engine_fork,
     additional_constraints_event_engine_fork, nullptr, 0, true, false},
    {"event_engine_listener", description_event_engine_listener,
     additional_constraints_event_engine_listener, nullptr, 0, true, false},
    {"event_engine_callback_cq", description_event_engine_callback_cq,
     additional_constraints_event_engine_callback_cq,
     required_experiments_event_engine_callback_cq, 2, true, true},
    {"event_engine_for_all_other_endpoints",
     description_event_engine_for_all_other_endpoints,
     additional_constraints_event_engine_for_all_other_endpoints,
     required_experiments_event_engine_for_all_other_endpoints, 4, true, false},
    {"event_engine_poller_for_python",
     description_event_engine_poller_for_python,
     additional_constraints_event_engine_poller_for_python, nullptr, 0, true,
     true},
    {"fail_recv_metadata_on_deadline_exceeded",
     description_fail_recv_metadata_on_deadline_exceeded,
     additional_constraints_fail_recv_metadata_on_deadline_exceeded, nullptr, 0,
     false, false},
    {"free_large_allocator", description_free_large_allocator,
     additional_constraints_free_large_allocator, nullptr, 0, false, true},
    {"fuse_filters", description_fuse_filters,
     additional_constraints_fuse_filters, nullptr, 0, false, false},
    {"h2_max_deallocating_streams_headroom",
     description_h2_max_deallocating_streams_headroom,
     additional_constraints_h2_max_deallocating_streams_headroom, nullptr, 0,
     false, true},
    {"inproc_cancel_stream", description_inproc_cancel_stream,
     additional_constraints_inproc_cancel_stream, nullptr, 0, true, true},
    {"keep_alive_ping_timer_batch", description_keep_alive_ping_timer_batch,
     additional_constraints_keep_alive_ping_timer_batch, nullptr, 0, false,
     true},
    {"local_connector_secure", description_local_connector_secure,
     additional_constraints_local_connector_secure, nullptr, 0, false, true},
    {"max_inflight_pings_strict_limit",
     description_max_inflight_pings_strict_limit,
     additional_constraints_max_inflight_pings_strict_limit, nullptr, 0, true,
     true},
    {"memory_optimization_01", description_memory_optimization_01,
     additional_constraints_memory_optimization_01, nullptr, 0, false, false},
    {"memory_optimization_02", description_memory_optimization_02,
     additional_constraints_memory_optimization_02, nullptr, 0, false, false},
    {"message_size_refactoring", description_message_size_refactoring,
     additional_constraints_message_size_refactoring, nullptr, 0, false, true},
    {"metadata_outstanding_token_refactor",
     description_metadata_outstanding_token_refactor,
     additional_constraints_metadata_outstanding_token_refactor, nullptr, 0,
     false, true},
    {"metadata_publish_to_app_tag", description_metadata_publish_to_app_tag,
     additional_constraints_metadata_publish_to_app_tag, nullptr, 0, true,
     true},
    {"monitoring_experiment", description_monitoring_experiment,
     additional_constraints_monitoring_experiment, nullptr, 0, true, true},
    {"multiping", description_multiping, additional_constraints_multiping,
     nullptr, 0, false, true},
    {"optimization_01", description_optimization_01,
     additional_constraints_optimization_01, nullptr, 0, true, true},
    {"optimization_02", description_optimization_02,
     additional_constraints_optimization_02, nullptr, 0, true, true},
    {"optimization_03", description_optimization_03,
     additional_constraints_optimization_03, nullptr, 0, true, true},
    {"optimization_04", description_optimization_04,
     additional_constraints_optimization_04, nullptr, 0, true, true},
    {"otel_export_telemetry_domains", description_otel_export_telemetry_domains,
     additional_constraints_otel_export_telemetry_domains, nullptr, 0, false,
     true},
    {"ph2_client", description_ph2_client, additional_constraints_ph2_client,
     nullptr, 0, false, true},
    {"ph2_client_server", description_ph2_client_server,
     additional_constraints_ph2_client_server, nullptr, 0, false, true},
    {"ph2_perf_01", description_ph2_perf_01, additional_constraints_ph2_perf_01,
     nullptr, 0, false, true},
    {"ph2_server", description_ph2_server, additional_constraints_ph2_server,
     nullptr, 0, false, true},
    {"pick_first_ignore_empty_updates",
     description_pick_first_ignore_empty_updates,
     additional_constraints_pick_first_ignore_empty_updates, nullptr, 0, false,
     true},
    {"pipelined_read_secure_endpoint",
     description_pipelined_read_secure_endpoint,
     additional_constraints_pipelined_read_secure_endpoint,
     required_experiments_pipelined_read_secure_endpoint, 2, false, false},
    {"pollset_alternative", description_pollset_alternative,
     additional_constraints_pollset_alternative,
     required_experiments_pollset_alternative, 2, false, false},
    {"prioritize_finished_requests", description_prioritize_finished_requests,
     additional_constraints_prioritize_finished_requests, nullptr, 0, false,
     true},
    {"promise_based_inproc_transport",
     description_promise_based_inproc_transport,
     additional_constraints_promise_based_inproc_transport, nullptr, 0, false,
     false},
    {"promise_batch_cleanup_on_cancel",
     description_promise_batch_cleanup_on_cancel,
     additional_constraints_promise_batch_cleanup_on_cancel, nullptr, 0, true,
     true},
    {"promise_filter_send_cancel_metadata",
     description_promise_filter_send_cancel_metadata,
     additional_constraints_promise_filter_send_cancel_metadata, nullptr, 0,
     false, true},
    {"retry_in_callv3", description_retry_in_callv3,
     additional_constraints_retry_in_callv3, nullptr, 0, false, true},
    {"return_preexisting_errors", description_return_preexisting_errors,
     additional_constraints_return_preexisting_errors, nullptr, 0, false, true},
    {"schedule_cancellation_over_write",
     description_schedule_cancellation_over_write,
     additional_constraints_schedule_cancellation_over_write, nullptr, 0, false,
     true},
    {"secure_endpoint_offload_large_reads",
     description_secure_endpoint_offload_large_reads,
     additional_constraints_secure_endpoint_offload_large_reads,
     required_experiments_secure_endpoint_offload_large_reads, 2, false, true},
    {"secure_endpoint_offload_large_writes",
     description_secure_endpoint_offload_large_writes,
     additional_constraints_secure_endpoint_offload_large_writes,
     required_experiments_secure_endpoint_offload_large_writes, 2, false, true},
    {"secure_endpoint_read_coalescing",
     description_secure_endpoint_read_coalescing,
     additional_constraints_secure_endpoint_read_coalescing,
     required_experiments_secure_endpoint_read_coalescing, 2, false, true},
    {"skip_clear_peer_on_cancellation",
     description_skip_clear_peer_on_cancellation,
     additional_constraints_skip_clear_peer_on_cancellation, nullptr, 0, false,
     true},
    {"subchannel_connection_scaling", description_subchannel_connection_scaling,
     additional_constraints_subchannel_connection_scaling, nullptr, 0, false,
     true},
    {"subchannel_wrapper_cleanup_on_orphan",
     description_subchannel_wrapper_cleanup_on_orphan,
     additional_constraints_subchannel_wrapper_cleanup_on_orphan, nullptr, 0,
     true, true},
    {"tcp_frame_size_tuning", description_tcp_frame_size_tuning,
     additional_constraints_tcp_frame_size_tuning, nullptr, 0, false, true},
    {"tcp_rcv_lowat", description_tcp_rcv_lowat,
     additional_constraints_tcp_rcv_lowat, nullptr, 0, false, true},
    {"tsi_frame_protector_without_locks",
     description_tsi_frame_protector_without_locks,
     additional_constraints_tsi_frame_protector_without_locks, nullptr, 0,
     false, true},
    {"unconstrained_max_quota_buffer_size",
     description_unconstrained_max_quota_buffer_size,
     additional_constraints_unconstrained_max_quota_buffer_size, nullptr, 0,
     false, true},
    {"use_call_event_engine_in_completion_queue",
     description_use_call_event_engine_in_completion_queue,
     additional_constraints_use_call_event_engine_in_completion_queue, nullptr,
     0, false, true},
    {"v2_non_owning_waker_implementation",
     description_v2_non_owning_waker_implementation,
     additional_constraints_v2_non_owning_waker_implementation, nullptr, 0,
     false, true},
    {"verbose_channelz_connection_logging",
     description_verbose_channelz_connection_logging,
     additional_constraints_verbose_channelz_connection_logging, nullptr, 0,
     false, true},
    {"wildcard_ip_expansion_restriction",
     description_wildcard_ip_expansion_restriction,
     additional_constraints_wildcard_ip_expansion_restriction, nullptr, 0,
     false, true},
    {"xds_server_filter_chain_per_route",
     description_xds_server_filter_chain_per_route,
     additional_constraints_xds_server_filter_chain_per_route, nullptr, 0,
     false, true},
};

}  // namespace grpc_core

#elif defined(GPR_WINDOWS)
namespace {
const char* const description_buffer_list_deletion_prep =
    "Gate the removal of old TCP timestamp collection mechanism.";
const char* const additional_constraints_buffer_list_deletion_prep = "{}";
const char* const description_call_tracer_in_transport =
    "Transport directly passes byte counts to CallTracer.";
const char* const additional_constraints_call_tracer_in_transport = "{}";
const char* const
    description_call_tracer_send_initial_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_initial_metadata_is_an_annotation =
        "{}";
const char* const
    description_call_tracer_send_trailing_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation =
        "{}";
const char* const description_callv3_batch_validation =
    "If enabled, perform validations on receiving a new batch.";
const char* const additional_constraints_callv3_batch_validation = "{}";
const char* const description_chaotic_good_framing_layer =
    "Enable the chaotic good framing layer.";
const char* const additional_constraints_chaotic_good_framing_layer = "{}";
const char* const description_chaotic_good_send_supported_features =
    "Send supported features in chaotic good settings frames.";
const char* const additional_constraints_chaotic_good_send_supported_features =
    "{}";
const char* const description_custom_frame_check =
    "Adding checks to custom HTTP2 frame.";
const char* const additional_constraints_custom_frame_check = "{}";
const char* const description_error_flatten =
    "Flatten errors to ordinary absl::Status form.";
const char* const additional_constraints_error_flatten = "{}";
const char* const description_event_engine_client =
    "Use EventEngine clients instead of iomgr's grpc_tcp_client";
const char* const additional_constraints_event_engine_client = "{}";
const char* const description_event_engine_dns =
    "If set, use EventEngine DNSResolver for client channel resolution";
const char* const additional_constraints_event_engine_dns = "{}";
const char* const description_event_engine_dns_non_client_channel =
    "If set, use EventEngine DNSResolver in other places besides client "
    "channel.";
const char* const additional_constraints_event_engine_dns_non_client_channel =
    "{}";
const char* const description_event_engine_fork =
    "Enables event engine fork handling, including onfork events and file "
    "descriptor generations";
const char* const additional_constraints_event_engine_fork = "{}";
const char* const description_event_engine_listener =
    "Use EventEngine listeners instead of iomgr's grpc_tcp_server";
const char* const additional_constraints_event_engine_listener = "{}";
const char* const description_event_engine_callback_cq =
    "Use EventEngine instead of the CallbackAlternativeCQ.";
const char* const additional_constraints_event_engine_callback_cq = "{}";
const uint8_t required_experiments_event_engine_callback_cq[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_for_all_other_endpoints =
    "Use EventEngine endpoints for all call sites, including direct uses of "
    "grpc_tcp_create.";
const char* const additional_constraints_event_engine_for_all_other_endpoints =
    "{}";
const uint8_t required_experiments_event_engine_for_all_other_endpoints[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineDns),
    static_cast<uint8_t>(
        grpc_core::kExperimentIdEventEngineDnsNonClientChannel),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_poller_for_python =
    "Enable event engine poller in gRPC Python";
const char* const additional_constraints_event_engine_poller_for_python = "{}";
const char* const description_fail_recv_metadata_on_deadline_exceeded =
    "Fail recv initial metadata when the deadline is exceeded.";
const char* const
    additional_constraints_fail_recv_metadata_on_deadline_exceeded = "{}";
const char* const description_free_large_allocator =
    "If set, return all free bytes from a \042big\042 allocator";
const char* const additional_constraints_free_large_allocator = "{}";
const char* const description_fuse_filters =
    "If set, individual filters are merged into fused filters";
const char* const additional_constraints_fuse_filters = "{}";
const char* const description_h2_max_deallocating_streams_headroom =
    "Separate allocated max concurrent streams.";
const char* const additional_constraints_h2_max_deallocating_streams_headroom =
    "{}";
const char* const description_inproc_cancel_stream =
    "If set, cancel inproc stream inside the transport mutex.";
const char* const additional_constraints_inproc_cancel_stream = "{}";
const char* const description_keep_alive_ping_timer_batch =
    "Avoid explicitly cancelling the keepalive timer. Instead adjust the "
    "callback to re-schedule itself to the next ping interval.";
const char* const additional_constraints_keep_alive_ping_timer_batch = "{}";
const char* const description_local_connector_secure =
    "Local security connector uses TSI_SECURITY_NONE for LOCAL_TCP "
    "connections.";
const char* const additional_constraints_local_connector_secure = "{}";
const char* const description_max_inflight_pings_strict_limit =
    "If set, the max inflight pings limit is strictly enforced.";
const char* const additional_constraints_max_inflight_pings_strict_limit = "{}";
const char* const description_memory_optimization_01 = "Memory Optimization";
const char* const additional_constraints_memory_optimization_01 = "{}";
const char* const description_memory_optimization_02 = "Memory Optimization 02";
const char* const additional_constraints_memory_optimization_02 = "{}";
const char* const description_message_size_refactoring =
    "Refactoring the message size config parsing.";
const char* const additional_constraints_message_size_refactoring = "{}";
const char* const description_metadata_outstanding_token_refactor =
    "Refactor ClientInitialMetadataOutstandingToken.";
const char* const additional_constraints_metadata_outstanding_token_refactor =
    "{}";
const char* const description_metadata_publish_to_app_tag =
    "Publish metadata to the app using the kPublishToApp metadata field.";
const char* const additional_constraints_metadata_publish_to_app_tag = "{}";
const char* const description_monitoring_experiment =
    "Placeholder experiment to prove/disprove our monitoring is working";
const char* const additional_constraints_monitoring_experiment = "{}";
const char* const description_multiping =
    "Allow more than one ping to be in flight at a time by default.";
const char* const additional_constraints_multiping = "{}";
const char* const description_optimization_01 = "Optimization";
const char* const additional_constraints_optimization_01 = "{}";
const char* const description_optimization_02 = "Optimization";
const char* const additional_constraints_optimization_02 = "{}";
const char* const description_optimization_03 = "Optimization";
const char* const additional_constraints_optimization_03 = "{}";
const char* const description_optimization_04 = "Optimization";
const char* const additional_constraints_optimization_04 = "{}";
const char* const description_otel_export_telemetry_domains =
    "Export telemetry domains in OpenTelemetry metrics.";
const char* const additional_constraints_otel_export_telemetry_domains = "{}";
const char* const description_ph2_client =
    "Use promises for the http2 client transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test PH2 Client "
    "with CHTTP2 Server.";
const char* const additional_constraints_ph2_client = "{}";
const char* const description_ph2_client_server =
    "This is a combination of ph2_server and ph2_client experiments. When this "
    "flag is enabled for the test suites, it would test PH2 Client with PH2 "
    "Server. We will NEVER be rolling out this experiment, because the roll "
    "out would be done using ph2_server and ph2_client. This experiment MUST "
    "be deleted when either ph2_server or ph2_client experiments are deleted.";
const char* const additional_constraints_ph2_client_server = "{}";
const char* const description_ph2_perf_01 =
    "Different performance experiments for PH2 Transport";
const char* const additional_constraints_ph2_perf_01 = "{}";
const char* const description_ph2_server =
    "Use promises for the http2 server transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test CHTTP2 "
    "Client with PH2 Server.";
const char* const additional_constraints_ph2_server = "{}";
const char* const description_pick_first_ignore_empty_updates =
    "Ignore empty resolutions in pick_first (delete)";
const char* const additional_constraints_pick_first_ignore_empty_updates = "{}";
const char* const description_pipelined_read_secure_endpoint =
    "Enable pipelined reads for EventEngine secure endpoints";
const char* const additional_constraints_pipelined_read_secure_endpoint = "{}";
const uint8_t required_experiments_pipelined_read_secure_endpoint[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_pollset_alternative =
    "Code outside iomgr that relies directly on pollsets will use non-pollset "
    "alternatives when enabled.";
const char* const additional_constraints_pollset_alternative = "{}";
const uint8_t required_experiments_pollset_alternative[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_prioritize_finished_requests =
    "Prioritize flushing out finished requests over other in-flight requests "
    "during transport writes.";
const char* const additional_constraints_prioritize_finished_requests = "{}";
const char* const description_promise_based_inproc_transport =
    "Use call-v3 for the in-process transport.";
const char* const additional_constraints_promise_based_inproc_transport = "{}";
const char* const description_promise_batch_cleanup_on_cancel =
    "Fix completion queue tag handling during promise batch cancellation.";
const char* const additional_constraints_promise_batch_cleanup_on_cancel = "{}";
const char* const description_promise_filter_send_cancel_metadata =
    "Enables sending all trailing metadata fields from server side "
    "promise-based filters upon stream cancellation.";
const char* const additional_constraints_promise_filter_send_cancel_metadata =
    "{}";
const char* const description_retry_in_callv3 = "Support retries with call-v3";
const char* const additional_constraints_retry_in_callv3 = "{}";
const char* const description_return_preexisting_errors =
    "Return errors that exist before the start of the call in RunHandler.";
const char* const additional_constraints_return_preexisting_errors = "{}";
const char* const description_schedule_cancellation_over_write =
    "Allow cancellation op to be scheduled over a write";
const char* const additional_constraints_schedule_cancellation_over_write =
    "{}";
const char* const description_secure_endpoint_offload_large_reads =
    "If a large read needs to be decrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_reads =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_reads[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_offload_large_writes =
    "If a large read write to be encrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_writes =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_writes[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_read_coalescing =
    "Coalesce decrypted bytes in SecureEndpoint if a read_hint is provided.";
const char* const additional_constraints_secure_endpoint_read_coalescing = "{}";
const uint8_t required_experiments_secure_endpoint_read_coalescing[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_skip_clear_peer_on_cancellation =
    "If set, skips clearing of peer string on call cancellation.";
const char* const additional_constraints_skip_clear_peer_on_cancellation = "{}";
const char* const description_subchannel_connection_scaling =
    "Subchannel connection scaling support.";
const char* const additional_constraints_subchannel_connection_scaling = "{}";
const char* const description_subchannel_wrapper_cleanup_on_orphan =
    "Fixes the subchannel wrapper to drop any non-cancelled watchers when it "
    "gets orphaned.";
const char* const additional_constraints_subchannel_wrapper_cleanup_on_orphan =
    "{}";
const char* const description_tcp_frame_size_tuning =
    "If set, enables TCP to use RPC size estimation made by higher layers. TCP "
    "would not indicate completion of a read operation until a specified "
    "number of bytes have been read over the socket. Buffers are also "
    "allocated according to estimated RPC sizes.";
const char* const additional_constraints_tcp_frame_size_tuning = "{}";
const char* const description_tcp_rcv_lowat =
    "Use SO_RCVLOWAT to avoid wakeups on the read path.";
const char* const additional_constraints_tcp_rcv_lowat = "{}";
const char* const description_tsi_frame_protector_without_locks =
    "Do not hold locks while using the tsi_frame_protector.";
const char* const additional_constraints_tsi_frame_protector_without_locks =
    "{}";
const char* const description_unconstrained_max_quota_buffer_size =
    "Discard the cap on the max free pool size for one memory allocator";
const char* const additional_constraints_unconstrained_max_quota_buffer_size =
    "{}";
const char* const description_use_call_event_engine_in_completion_queue =
    "Use the call event engine to run callbacks in completion queue.";
const char* const
    additional_constraints_use_call_event_engine_in_completion_queue = "{}";
const char* const description_v2_non_owning_waker_implementation =
    "Use non-owning wakers in v2 filter bridge.";
const char* const additional_constraints_v2_non_owning_waker_implementation =
    "{}";
const char* const description_verbose_channelz_connection_logging =
    "Verbose logging of events into channelz during connection setup.";
const char* const additional_constraints_verbose_channelz_connection_logging =
    "{}";
const char* const description_wildcard_ip_expansion_restriction =
    "If set, adds optional restriction on when to expand wildcard IPs.";
const char* const additional_constraints_wildcard_ip_expansion_restriction =
    "{}";
const char* const description_xds_server_filter_chain_per_route =
    "xDS servers use a separate filter chain for each route.";
const char* const additional_constraints_xds_server_filter_chain_per_route =
    "{}";
}  // namespace

namespace grpc_core {

const ExperimentMetadata g_experiment_metadata[] = {
    {"buffer_list_deletion_prep", description_buffer_list_deletion_prep,
     additional_constraints_buffer_list_deletion_prep, nullptr, 0, false, true},
    {"call_tracer_in_transport", description_call_tracer_in_transport,
     additional_constraints_call_tracer_in_transport, nullptr, 0, true, false},
    {"call_tracer_send_initial_metadata_is_an_annotation",
     description_call_tracer_send_initial_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_initial_metadata_is_an_annotation,
     nullptr, 0, true, true},
    {"call_tracer_send_trailing_metadata_is_an_annotation",
     description_call_tracer_send_trailing_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation,
     nullptr, 0, false, true},
    {"callv3_batch_validation", description_callv3_batch_validation,
     additional_constraints_callv3_batch_validation, nullptr, 0, true, true},
    {"chaotic_good_framing_layer", description_chaotic_good_framing_layer,
     additional_constraints_chaotic_good_framing_layer, nullptr, 0, true,
     false},
    {"chaotic_good_send_supported_features",
     description_chaotic_good_send_supported_features,
     additional_constraints_chaotic_good_send_supported_features, nullptr, 0,
     false, true},
    {"custom_frame_check", description_custom_frame_check,
     additional_constraints_custom_frame_check, nullptr, 0, true, false},
    {"error_flatten", description_error_flatten,
     additional_constraints_error_flatten, nullptr, 0, true, false},
    {"event_engine_client", description_event_engine_client,
     additional_constraints_event_engine_client, nullptr, 0, true, false},
    {"event_engine_dns", description_event_engine_dns,
     additional_constraints_event_engine_dns, nullptr, 0, true, false},
    {"event_engine_dns_non_client_channel",
     description_event_engine_dns_non_client_channel,
     additional_constraints_event_engine_dns_non_client_channel, nullptr, 0,
     true, false},
    {"event_engine_fork", description_event_engine_fork,
     additional_constraints_event_engine_fork, nullptr, 0, true, false},
    {"event_engine_listener", description_event_engine_listener,
     additional_constraints_event_engine_listener, nullptr, 0, true, false},
    {"event_engine_callback_cq", description_event_engine_callback_cq,
     additional_constraints_event_engine_callback_cq,
     required_experiments_event_engine_callback_cq, 2, true, true},
    {"event_engine_for_all_other_endpoints",
     description_event_engine_for_all_other_endpoints,
     additional_constraints_event_engine_for_all_other_endpoints,
     required_experiments_event_engine_for_all_other_endpoints, 4, true, false},
    {"event_engine_poller_for_python",
     description_event_engine_poller_for_python,
     additional_constraints_event_engine_poller_for_python, nullptr, 0, true,
     true},
    {"fail_recv_metadata_on_deadline_exceeded",
     description_fail_recv_metadata_on_deadline_exceeded,
     additional_constraints_fail_recv_metadata_on_deadline_exceeded, nullptr, 0,
     false, false},
    {"free_large_allocator", description_free_large_allocator,
     additional_constraints_free_large_allocator, nullptr, 0, false, true},
    {"fuse_filters", description_fuse_filters,
     additional_constraints_fuse_filters, nullptr, 0, false, false},
    {"h2_max_deallocating_streams_headroom",
     description_h2_max_deallocating_streams_headroom,
     additional_constraints_h2_max_deallocating_streams_headroom, nullptr, 0,
     false, true},
    {"inproc_cancel_stream", description_inproc_cancel_stream,
     additional_constraints_inproc_cancel_stream, nullptr, 0, true, true},
    {"keep_alive_ping_timer_batch", description_keep_alive_ping_timer_batch,
     additional_constraints_keep_alive_ping_timer_batch, nullptr, 0, false,
     true},
    {"local_connector_secure", description_local_connector_secure,
     additional_constraints_local_connector_secure, nullptr, 0, false, true},
    {"max_inflight_pings_strict_limit",
     description_max_inflight_pings_strict_limit,
     additional_constraints_max_inflight_pings_strict_limit, nullptr, 0, true,
     true},
    {"memory_optimization_01", description_memory_optimization_01,
     additional_constraints_memory_optimization_01, nullptr, 0, false, false},
    {"memory_optimization_02", description_memory_optimization_02,
     additional_constraints_memory_optimization_02, nullptr, 0, false, false},
    {"message_size_refactoring", description_message_size_refactoring,
     additional_constraints_message_size_refactoring, nullptr, 0, false, true},
    {"metadata_outstanding_token_refactor",
     description_metadata_outstanding_token_refactor,
     additional_constraints_metadata_outstanding_token_refactor, nullptr, 0,
     false, true},
    {"metadata_publish_to_app_tag", description_metadata_publish_to_app_tag,
     additional_constraints_metadata_publish_to_app_tag, nullptr, 0, true,
     true},
    {"monitoring_experiment", description_monitoring_experiment,
     additional_constraints_monitoring_experiment, nullptr, 0, true, true},
    {"multiping", description_multiping, additional_constraints_multiping,
     nullptr, 0, false, true},
    {"optimization_01", description_optimization_01,
     additional_constraints_optimization_01, nullptr, 0, true, true},
    {"optimization_02", description_optimization_02,
     additional_constraints_optimization_02, nullptr, 0, true, true},
    {"optimization_03", description_optimization_03,
     additional_constraints_optimization_03, nullptr, 0, true, true},
    {"optimization_04", description_optimization_04,
     additional_constraints_optimization_04, nullptr, 0, true, true},
    {"otel_export_telemetry_domains", description_otel_export_telemetry_domains,
     additional_constraints_otel_export_telemetry_domains, nullptr, 0, false,
     true},
    {"ph2_client", description_ph2_client, additional_constraints_ph2_client,
     nullptr, 0, false, true},
    {"ph2_client_server", description_ph2_client_server,
     additional_constraints_ph2_client_server, nullptr, 0, false, true},
    {"ph2_perf_01", description_ph2_perf_01, additional_constraints_ph2_perf_01,
     nullptr, 0, false, true},
    {"ph2_server", description_ph2_server, additional_constraints_ph2_server,
     nullptr, 0, false, true},
    {"pick_first_ignore_empty_updates",
     description_pick_first_ignore_empty_updates,
     additional_constraints_pick_first_ignore_empty_updates, nullptr, 0, false,
     true},
    {"pipelined_read_secure_endpoint",
     description_pipelined_read_secure_endpoint,
     additional_constraints_pipelined_read_secure_endpoint,
     required_experiments_pipelined_read_secure_endpoint, 2, false, false},
    {"pollset_alternative", description_pollset_alternative,
     additional_constraints_pollset_alternative,
     required_experiments_pollset_alternative, 2, false, false},
    {"prioritize_finished_requests", description_prioritize_finished_requests,
     additional_constraints_prioritize_finished_requests, nullptr, 0, false,
     true},
    {"promise_based_inproc_transport",
     description_promise_based_inproc_transport,
     additional_constraints_promise_based_inproc_transport, nullptr, 0, false,
     false},
    {"promise_batch_cleanup_on_cancel",
     description_promise_batch_cleanup_on_cancel,
     additional_constraints_promise_batch_cleanup_on_cancel, nullptr, 0, true,
     true},
    {"promise_filter_send_cancel_metadata",
     description_promise_filter_send_cancel_metadata,
     additional_constraints_promise_filter_send_cancel_metadata, nullptr, 0,
     false, true},
    {"retry_in_callv3", description_retry_in_callv3,
     additional_constraints_retry_in_callv3, nullptr, 0, false, true},
    {"return_preexisting_errors", description_return_preexisting_errors,
     additional_constraints_return_preexisting_errors, nullptr, 0, false, true},
    {"schedule_cancellation_over_write",
     description_schedule_cancellation_over_write,
     additional_constraints_schedule_cancellation_over_write, nullptr, 0, false,
     true},
    {"secure_endpoint_offload_large_reads",
     description_secure_endpoint_offload_large_reads,
     additional_constraints_secure_endpoint_offload_large_reads,
     required_experiments_secure_endpoint_offload_large_reads, 2, false, true},
    {"secure_endpoint_offload_large_writes",
     description_secure_endpoint_offload_large_writes,
     additional_constraints_secure_endpoint_offload_large_writes,
     required_experiments_secure_endpoint_offload_large_writes, 2, false, true},
    {"secure_endpoint_read_coalescing",
     description_secure_endpoint_read_coalescing,
     additional_constraints_secure_endpoint_read_coalescing,
     required_experiments_secure_endpoint_read_coalescing, 2, false, true},
    {"skip_clear_peer_on_cancellation",
     description_skip_clear_peer_on_cancellation,
     additional_constraints_skip_clear_peer_on_cancellation, nullptr, 0, false,
     true},
    {"subchannel_connection_scaling", description_subchannel_connection_scaling,
     additional_constraints_subchannel_connection_scaling, nullptr, 0, false,
     true},
    {"subchannel_wrapper_cleanup_on_orphan",
     description_subchannel_wrapper_cleanup_on_orphan,
     additional_constraints_subchannel_wrapper_cleanup_on_orphan, nullptr, 0,
     true, true},
    {"tcp_frame_size_tuning", description_tcp_frame_size_tuning,
     additional_constraints_tcp_frame_size_tuning, nullptr, 0, false, true},
    {"tcp_rcv_lowat", description_tcp_rcv_lowat,
     additional_constraints_tcp_rcv_lowat, nullptr, 0, false, true},
    {"tsi_frame_protector_without_locks",
     description_tsi_frame_protector_without_locks,
     additional_constraints_tsi_frame_protector_without_locks, nullptr, 0,
     false, true},
    {"unconstrained_max_quota_buffer_size",
     description_unconstrained_max_quota_buffer_size,
     additional_constraints_unconstrained_max_quota_buffer_size, nullptr, 0,
     false, true},
    {"use_call_event_engine_in_completion_queue",
     description_use_call_event_engine_in_completion_queue,
     additional_constraints_use_call_event_engine_in_completion_queue, nullptr,
     0, false, true},
    {"v2_non_owning_waker_implementation",
     description_v2_non_owning_waker_implementation,
     additional_constraints_v2_non_owning_waker_implementation, nullptr, 0,
     false, true},
    {"verbose_channelz_connection_logging",
     description_verbose_channelz_connection_logging,
     additional_constraints_verbose_channelz_connection_logging, nullptr, 0,
     false, true},
    {"wildcard_ip_expansion_restriction",
     description_wildcard_ip_expansion_restriction,
     additional_constraints_wildcard_ip_expansion_restriction, nullptr, 0,
     false, true},
    {"xds_server_filter_chain_per_route",
     description_xds_server_filter_chain_per_route,
     additional_constraints_xds_server_filter_chain_per_route, nullptr, 0,
     false, true},
};

}  // namespace grpc_core

#else
namespace {
const char* const description_buffer_list_deletion_prep =
    "Gate the removal of old TCP timestamp collection mechanism.";
const char* const additional_constraints_buffer_list_deletion_prep = "{}";
const char* const description_call_tracer_in_transport =
    "Transport directly passes byte counts to CallTracer.";
const char* const additional_constraints_call_tracer_in_transport = "{}";
const char* const
    description_call_tracer_send_initial_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_initial_metadata_is_an_annotation =
        "{}";
const char* const
    description_call_tracer_send_trailing_metadata_is_an_annotation =
        "Use the new annotation-based CallTracer API.";
const char* const
    additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation =
        "{}";
const char* const description_callv3_batch_validation =
    "If enabled, perform validations on receiving a new batch.";
const char* const additional_constraints_callv3_batch_validation = "{}";
const char* const description_chaotic_good_framing_layer =
    "Enable the chaotic good framing layer.";
const char* const additional_constraints_chaotic_good_framing_layer = "{}";
const char* const description_chaotic_good_send_supported_features =
    "Send supported features in chaotic good settings frames.";
const char* const additional_constraints_chaotic_good_send_supported_features =
    "{}";
const char* const description_custom_frame_check =
    "Adding checks to custom HTTP2 frame.";
const char* const additional_constraints_custom_frame_check = "{}";
const char* const description_error_flatten =
    "Flatten errors to ordinary absl::Status form.";
const char* const additional_constraints_error_flatten = "{}";
const char* const description_event_engine_client =
    "Use EventEngine clients instead of iomgr's grpc_tcp_client";
const char* const additional_constraints_event_engine_client = "{}";
const char* const description_event_engine_dns =
    "If set, use EventEngine DNSResolver for client channel resolution";
const char* const additional_constraints_event_engine_dns = "{}";
const char* const description_event_engine_dns_non_client_channel =
    "If set, use EventEngine DNSResolver in other places besides client "
    "channel.";
const char* const additional_constraints_event_engine_dns_non_client_channel =
    "{}";
const char* const description_event_engine_fork =
    "Enables event engine fork handling, including onfork events and file "
    "descriptor generations";
const char* const additional_constraints_event_engine_fork = "{}";
const char* const description_event_engine_listener =
    "Use EventEngine listeners instead of iomgr's grpc_tcp_server";
const char* const additional_constraints_event_engine_listener = "{}";
const char* const description_event_engine_callback_cq =
    "Use EventEngine instead of the CallbackAlternativeCQ.";
const char* const additional_constraints_event_engine_callback_cq = "{}";
const uint8_t required_experiments_event_engine_callback_cq[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_for_all_other_endpoints =
    "Use EventEngine endpoints for all call sites, including direct uses of "
    "grpc_tcp_create.";
const char* const additional_constraints_event_engine_for_all_other_endpoints =
    "{}";
const uint8_t required_experiments_event_engine_for_all_other_endpoints[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineDns),
    static_cast<uint8_t>(
        grpc_core::kExperimentIdEventEngineDnsNonClientChannel),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_event_engine_poller_for_python =
    "Enable event engine poller in gRPC Python";
const char* const additional_constraints_event_engine_poller_for_python = "{}";
const char* const description_fail_recv_metadata_on_deadline_exceeded =
    "Fail recv initial metadata when the deadline is exceeded.";
const char* const
    additional_constraints_fail_recv_metadata_on_deadline_exceeded = "{}";
const char* const description_free_large_allocator =
    "If set, return all free bytes from a \042big\042 allocator";
const char* const additional_constraints_free_large_allocator = "{}";
const char* const description_fuse_filters =
    "If set, individual filters are merged into fused filters";
const char* const additional_constraints_fuse_filters = "{}";
const char* const description_h2_max_deallocating_streams_headroom =
    "Separate allocated max concurrent streams.";
const char* const additional_constraints_h2_max_deallocating_streams_headroom =
    "{}";
const char* const description_inproc_cancel_stream =
    "If set, cancel inproc stream inside the transport mutex.";
const char* const additional_constraints_inproc_cancel_stream = "{}";
const char* const description_keep_alive_ping_timer_batch =
    "Avoid explicitly cancelling the keepalive timer. Instead adjust the "
    "callback to re-schedule itself to the next ping interval.";
const char* const additional_constraints_keep_alive_ping_timer_batch = "{}";
const char* const description_local_connector_secure =
    "Local security connector uses TSI_SECURITY_NONE for LOCAL_TCP "
    "connections.";
const char* const additional_constraints_local_connector_secure = "{}";
const char* const description_max_inflight_pings_strict_limit =
    "If set, the max inflight pings limit is strictly enforced.";
const char* const additional_constraints_max_inflight_pings_strict_limit = "{}";
const char* const description_memory_optimization_01 = "Memory Optimization";
const char* const additional_constraints_memory_optimization_01 = "{}";
const char* const description_memory_optimization_02 = "Memory Optimization 02";
const char* const additional_constraints_memory_optimization_02 = "{}";
const char* const description_message_size_refactoring =
    "Refactoring the message size config parsing.";
const char* const additional_constraints_message_size_refactoring = "{}";
const char* const description_metadata_outstanding_token_refactor =
    "Refactor ClientInitialMetadataOutstandingToken.";
const char* const additional_constraints_metadata_outstanding_token_refactor =
    "{}";
const char* const description_metadata_publish_to_app_tag =
    "Publish metadata to the app using the kPublishToApp metadata field.";
const char* const additional_constraints_metadata_publish_to_app_tag = "{}";
const char* const description_monitoring_experiment =
    "Placeholder experiment to prove/disprove our monitoring is working";
const char* const additional_constraints_monitoring_experiment = "{}";
const char* const description_multiping =
    "Allow more than one ping to be in flight at a time by default.";
const char* const additional_constraints_multiping = "{}";
const char* const description_optimization_01 = "Optimization";
const char* const additional_constraints_optimization_01 = "{}";
const char* const description_optimization_02 = "Optimization";
const char* const additional_constraints_optimization_02 = "{}";
const char* const description_optimization_03 = "Optimization";
const char* const additional_constraints_optimization_03 = "{}";
const char* const description_optimization_04 = "Optimization";
const char* const additional_constraints_optimization_04 = "{}";
const char* const description_otel_export_telemetry_domains =
    "Export telemetry domains in OpenTelemetry metrics.";
const char* const additional_constraints_otel_export_telemetry_domains = "{}";
const char* const description_ph2_client =
    "Use promises for the http2 client transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test PH2 Client "
    "with CHTTP2 Server.";
const char* const additional_constraints_ph2_client = "{}";
const char* const description_ph2_client_server =
    "This is a combination of ph2_server and ph2_client experiments. When this "
    "flag is enabled for the test suites, it would test PH2 Client with PH2 "
    "Server. We will NEVER be rolling out this experiment, because the roll "
    "out would be done using ph2_server and ph2_client. This experiment MUST "
    "be deleted when either ph2_server or ph2_client experiments are deleted.";
const char* const additional_constraints_ph2_client_server = "{}";
const char* const description_ph2_perf_01 =
    "Different performance experiments for PH2 Transport";
const char* const additional_constraints_ph2_perf_01 = "{}";
const char* const description_ph2_server =
    "Use promises for the http2 server transport. We have kept client and "
    "server transport experiments separate to help with smoother roll outs. "
    "When this flag is enabled for the test suites, it would test CHTTP2 "
    "Client with PH2 Server.";
const char* const additional_constraints_ph2_server = "{}";
const char* const description_pick_first_ignore_empty_updates =
    "Ignore empty resolutions in pick_first (delete)";
const char* const additional_constraints_pick_first_ignore_empty_updates = "{}";
const char* const description_pipelined_read_secure_endpoint =
    "Enable pipelined reads for EventEngine secure endpoints";
const char* const additional_constraints_pipelined_read_secure_endpoint = "{}";
const uint8_t required_experiments_pipelined_read_secure_endpoint[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_pollset_alternative =
    "Code outside iomgr that relies directly on pollsets will use non-pollset "
    "alternatives when enabled.";
const char* const additional_constraints_pollset_alternative = "{}";
const uint8_t required_experiments_pollset_alternative[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_prioritize_finished_requests =
    "Prioritize flushing out finished requests over other in-flight requests "
    "during transport writes.";
const char* const additional_constraints_prioritize_finished_requests = "{}";
const char* const description_promise_based_inproc_transport =
    "Use call-v3 for the in-process transport.";
const char* const additional_constraints_promise_based_inproc_transport = "{}";
const char* const description_promise_batch_cleanup_on_cancel =
    "Fix completion queue tag handling during promise batch cancellation.";
const char* const additional_constraints_promise_batch_cleanup_on_cancel = "{}";
const char* const description_promise_filter_send_cancel_metadata =
    "Enables sending all trailing metadata fields from server side "
    "promise-based filters upon stream cancellation.";
const char* const additional_constraints_promise_filter_send_cancel_metadata =
    "{}";
const char* const description_retry_in_callv3 = "Support retries with call-v3";
const char* const additional_constraints_retry_in_callv3 = "{}";
const char* const description_return_preexisting_errors =
    "Return errors that exist before the start of the call in RunHandler.";
const char* const additional_constraints_return_preexisting_errors = "{}";
const char* const description_schedule_cancellation_over_write =
    "Allow cancellation op to be scheduled over a write";
const char* const additional_constraints_schedule_cancellation_over_write =
    "{}";
const char* const description_secure_endpoint_offload_large_reads =
    "If a large read needs to be decrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_reads =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_reads[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_offload_large_writes =
    "If a large read write to be encrypted, use a separate thread.";
const char* const additional_constraints_secure_endpoint_offload_large_writes =
    "{}";
const uint8_t required_experiments_secure_endpoint_offload_large_writes[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_secure_endpoint_read_coalescing =
    "Coalesce decrypted bytes in SecureEndpoint if a read_hint is provided.";
const char* const additional_constraints_secure_endpoint_read_coalescing = "{}";
const uint8_t required_experiments_secure_endpoint_read_coalescing[] = {
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineClient),
    static_cast<uint8_t>(grpc_core::kExperimentIdEventEngineListener)};
const char* const description_skip_clear_peer_on_cancellation =
    "If set, skips clearing of peer string on call cancellation.";
const char* const additional_constraints_skip_clear_peer_on_cancellation = "{}";
const char* const description_subchannel_connection_scaling =
    "Subchannel connection scaling support.";
const char* const additional_constraints_subchannel_connection_scaling = "{}";
const char* const description_subchannel_wrapper_cleanup_on_orphan =
    "Fixes the subchannel wrapper to drop any non-cancelled watchers when it "
    "gets orphaned.";
const char* const additional_constraints_subchannel_wrapper_cleanup_on_orphan =
    "{}";
const char* const description_tcp_frame_size_tuning =
    "If set, enables TCP to use RPC size estimation made by higher layers. TCP "
    "would not indicate completion of a read operation until a specified "
    "number of bytes have been read over the socket. Buffers are also "
    "allocated according to estimated RPC sizes.";
const char* const additional_constraints_tcp_frame_size_tuning = "{}";
const char* const description_tcp_rcv_lowat =
    "Use SO_RCVLOWAT to avoid wakeups on the read path.";
const char* const additional_constraints_tcp_rcv_lowat = "{}";
const char* const description_tsi_frame_protector_without_locks =
    "Do not hold locks while using the tsi_frame_protector.";
const char* const additional_constraints_tsi_frame_protector_without_locks =
    "{}";
const char* const description_unconstrained_max_quota_buffer_size =
    "Discard the cap on the max free pool size for one memory allocator";
const char* const additional_constraints_unconstrained_max_quota_buffer_size =
    "{}";
const char* const description_use_call_event_engine_in_completion_queue =
    "Use the call event engine to run callbacks in completion queue.";
const char* const
    additional_constraints_use_call_event_engine_in_completion_queue = "{}";
const char* const description_v2_non_owning_waker_implementation =
    "Use non-owning wakers in v2 filter bridge.";
const char* const additional_constraints_v2_non_owning_waker_implementation =
    "{}";
const char* const description_verbose_channelz_connection_logging =
    "Verbose logging of events into channelz during connection setup.";
const char* const additional_constraints_verbose_channelz_connection_logging =
    "{}";
const char* const description_wildcard_ip_expansion_restriction =
    "If set, adds optional restriction on when to expand wildcard IPs.";
const char* const additional_constraints_wildcard_ip_expansion_restriction =
    "{}";
const char* const description_xds_server_filter_chain_per_route =
    "xDS servers use a separate filter chain for each route.";
const char* const additional_constraints_xds_server_filter_chain_per_route =
    "{}";
}  // namespace

namespace grpc_core {

const ExperimentMetadata g_experiment_metadata[] = {
    {"buffer_list_deletion_prep", description_buffer_list_deletion_prep,
     additional_constraints_buffer_list_deletion_prep, nullptr, 0, false, true},
    {"call_tracer_in_transport", description_call_tracer_in_transport,
     additional_constraints_call_tracer_in_transport, nullptr, 0, true, false},
    {"call_tracer_send_initial_metadata_is_an_annotation",
     description_call_tracer_send_initial_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_initial_metadata_is_an_annotation,
     nullptr, 0, true, true},
    {"call_tracer_send_trailing_metadata_is_an_annotation",
     description_call_tracer_send_trailing_metadata_is_an_annotation,
     additional_constraints_call_tracer_send_trailing_metadata_is_an_annotation,
     nullptr, 0, false, true},
    {"callv3_batch_validation", description_callv3_batch_validation,
     additional_constraints_callv3_batch_validation, nullptr, 0, true, true},
    {"chaotic_good_framing_layer", description_chaotic_good_framing_layer,
     additional_constraints_chaotic_good_framing_layer, nullptr, 0, true,
     false},
    {"chaotic_good_send_supported_features",
     description_chaotic_good_send_supported_features,
     additional_constraints_chaotic_good_send_supported_features, nullptr, 0,
     false, true},
    {"custom_frame_check", description_custom_frame_check,
     additional_constraints_custom_frame_check, nullptr, 0, true, false},
    {"error_flatten", description_error_flatten,
     additional_constraints_error_flatten, nullptr, 0, true, false},
    {"event_engine_client", description_event_engine_client,
     additional_constraints_event_engine_client, nullptr, 0, true, false},
    {"event_engine_dns", description_event_engine_dns,
     additional_constraints_event_engine_dns, nullptr, 0, true, false},
    {"event_engine_dns_non_client_channel",
     description_event_engine_dns_non_client_channel,
     additional_constraints_event_engine_dns_non_client_channel, nullptr, 0,
     true, false},
    {"event_engine_fork", description_event_engine_fork,
     additional_constraints_event_engine_fork, nullptr, 0, true, false},
    {"event_engine_listener", description_event_engine_listener,
     additional_constraints_event_engine_listener, nullptr, 0, true, false},
    {"event_engine_callback_cq", description_event_engine_callback_cq,
     additional_constraints_event_engine_callback_cq,
     required_experiments_event_engine_callback_cq, 2, true, true},
    {"event_engine_for_all_other_endpoints",
     description_event_engine_for_all_other_endpoints,
     additional_constraints_event_engine_for_all_other_endpoints,
     required_experiments_event_engine_for_all_other_endpoints, 4, true, false},
    {"event_engine_poller_for_python",
     description_event_engine_poller_for_python,
     additional_constraints_event_engine_poller_for_python, nullptr, 0, true,
     true},
    {"fail_recv_metadata_on_deadline_exceeded",
     description_fail_recv_metadata_on_deadline_exceeded,
     additional_constraints_fail_recv_metadata_on_deadline_exceeded, nullptr, 0,
     false, false},
    {"free_large_allocator", description_free_large_allocator,
     additional_constraints_free_large_allocator, nullptr, 0, false, true},
    {"fuse_filters", description_fuse_filters,
     additional_constraints_fuse_filters, nullptr, 0, false, false},
    {"h2_max_deallocating_streams_headroom",
     description_h2_max_deallocating_streams_headroom,
     additional_constraints_h2_max_deallocating_streams_headroom, nullptr, 0,
     false, true},
    {"inproc_cancel_stream", description_inproc_cancel_stream,
     additional_constraints_inproc_cancel_stream, nullptr, 0, true, true},
    {"keep_alive_ping_timer_batch", description_keep_alive_ping_timer_batch,
     additional_constraints_keep_alive_ping_timer_batch, nullptr, 0, false,
     true},
    {"local_connector_secure", description_local_connector_secure,
     additional_constraints_local_connector_secure, nullptr, 0, false, true},
    {"max_inflight_pings_strict_limit",
     description_max_inflight_pings_strict_limit,
     additional_constraints_max_inflight_pings_strict_limit, nullptr, 0, true,
     true},
    {"memory_optimization_01", description_memory_optimization_01,
     additional_constraints_memory_optimization_01, nullptr, 0, false, false},
    {"memory_optimization_02", description_memory_optimization_02,
     additional_constraints_memory_optimization_02, nullptr, 0, false, false},
    {"message_size_refactoring", description_message_size_refactoring,
     additional_constraints_message_size_refactoring, nullptr, 0, false, true},
    {"metadata_outstanding_token_refactor",
     description_metadata_outstanding_token_refactor,
     additional_constraints_metadata_outstanding_token_refactor, nullptr, 0,
     false, true},
    {"metadata_publish_to_app_tag", description_metadata_publish_to_app_tag,
     additional_constraints_metadata_publish_to_app_tag, nullptr, 0, true,
     true},
    {"monitoring_experiment", description_monitoring_experiment,
     additional_constraints_monitoring_experiment, nullptr, 0, true, true},
    {"multiping", description_multiping, additional_constraints_multiping,
     nullptr, 0, false, true},
    {"optimization_01", description_optimization_01,
     additional_constraints_optimization_01, nullptr, 0, true, true},
    {"optimization_02", description_optimization_02,
     additional_constraints_optimization_02, nullptr, 0, true, true},
    {"optimization_03", description_optimization_03,
     additional_constraints_optimization_03, nullptr, 0, true, true},
    {"optimization_04", description_optimization_04,
     additional_constraints_optimization_04, nullptr, 0, true, true},
    {"otel_export_telemetry_domains", description_otel_export_telemetry_domains,
     additional_constraints_otel_export_telemetry_domains, nullptr, 0, false,
     true},
    {"ph2_client", description_ph2_client, additional_constraints_ph2_client,
     nullptr, 0, false, true},
    {"ph2_client_server", description_ph2_client_server,
     additional_constraints_ph2_client_server, nullptr, 0, false, true},
    {"ph2_perf_01", description_ph2_perf_01, additional_constraints_ph2_perf_01,
     nullptr, 0, false, true},
    {"ph2_server", description_ph2_server, additional_constraints_ph2_server,
     nullptr, 0, false, true},
    {"pick_first_ignore_empty_updates",
     description_pick_first_ignore_empty_updates,
     additional_constraints_pick_first_ignore_empty_updates, nullptr, 0, false,
     true},
    {"pipelined_read_secure_endpoint",
     description_pipelined_read_secure_endpoint,
     additional_constraints_pipelined_read_secure_endpoint,
     required_experiments_pipelined_read_secure_endpoint, 2, false, false},
    {"pollset_alternative", description_pollset_alternative,
     additional_constraints_pollset_alternative,
     required_experiments_pollset_alternative, 2, false, false},
    {"prioritize_finished_requests", description_prioritize_finished_requests,
     additional_constraints_prioritize_finished_requests, nullptr, 0, false,
     true},
    {"promise_based_inproc_transport",
     description_promise_based_inproc_transport,
     additional_constraints_promise_based_inproc_transport, nullptr, 0, false,
     false},
    {"promise_batch_cleanup_on_cancel",
     description_promise_batch_cleanup_on_cancel,
     additional_constraints_promise_batch_cleanup_on_cancel, nullptr, 0, true,
     true},
    {"promise_filter_send_cancel_metadata",
     description_promise_filter_send_cancel_metadata,
     additional_constraints_promise_filter_send_cancel_metadata, nullptr, 0,
     false, true},
    {"retry_in_callv3", description_retry_in_callv3,
     additional_constraints_retry_in_callv3, nullptr, 0, false, true},
    {"return_preexisting_errors", description_return_preexisting_errors,
     additional_constraints_return_preexisting_errors, nullptr, 0, false, true},
    {"schedule_cancellation_over_write",
     description_schedule_cancellation_over_write,
     additional_constraints_schedule_cancellation_over_write, nullptr, 0, false,
     true},
    {"secure_endpoint_offload_large_reads",
     description_secure_endpoint_offload_large_reads,
     additional_constraints_secure_endpoint_offload_large_reads,
     required_experiments_secure_endpoint_offload_large_reads, 2, false, true},
    {"secure_endpoint_offload_large_writes",
     description_secure_endpoint_offload_large_writes,
     additional_constraints_secure_endpoint_offload_large_writes,
     required_experiments_secure_endpoint_offload_large_writes, 2, false, true},
    {"secure_endpoint_read_coalescing",
     description_secure_endpoint_read_coalescing,
     additional_constraints_secure_endpoint_read_coalescing,
     required_experiments_secure_endpoint_read_coalescing, 2, false, true},
    {"skip_clear_peer_on_cancellation",
     description_skip_clear_peer_on_cancellation,
     additional_constraints_skip_clear_peer_on_cancellation, nullptr, 0, false,
     true},
    {"subchannel_connection_scaling", description_subchannel_connection_scaling,
     additional_constraints_subchannel_connection_scaling, nullptr, 0, false,
     true},
    {"subchannel_wrapper_cleanup_on_orphan",
     description_subchannel_wrapper_cleanup_on_orphan,
     additional_constraints_subchannel_wrapper_cleanup_on_orphan, nullptr, 0,
     true, true},
    {"tcp_frame_size_tuning", description_tcp_frame_size_tuning,
     additional_constraints_tcp_frame_size_tuning, nullptr, 0, false, true},
    {"tcp_rcv_lowat", description_tcp_rcv_lowat,
     additional_constraints_tcp_rcv_lowat, nullptr, 0, false, true},
    {"tsi_frame_protector_without_locks",
     description_tsi_frame_protector_without_locks,
     additional_constraints_tsi_frame_protector_without_locks, nullptr, 0,
     false, true},
    {"unconstrained_max_quota_buffer_size",
     description_unconstrained_max_quota_buffer_size,
     additional_constraints_unconstrained_max_quota_buffer_size, nullptr, 0,
     false, true},
    {"use_call_event_engine_in_completion_queue",
     description_use_call_event_engine_in_completion_queue,
     additional_constraints_use_call_event_engine_in_completion_queue, nullptr,
     0, false, true},
    {"v2_non_owning_waker_implementation",
     description_v2_non_owning_waker_implementation,
     additional_constraints_v2_non_owning_waker_implementation, nullptr, 0,
     false, true},
    {"verbose_channelz_connection_logging",
     description_verbose_channelz_connection_logging,
     additional_constraints_verbose_channelz_connection_logging, nullptr, 0,
     false, true},
    {"wildcard_ip_expansion_restriction",
     description_wildcard_ip_expansion_restriction,
     additional_constraints_wildcard_ip_expansion_restriction, nullptr, 0,
     false, true},
    {"xds_server_filter_chain_per_route",
     description_xds_server_filter_chain_per_route,
     additional_constraints_xds_server_filter_chain_per_route, nullptr, 0,
     false, true},
};

}  // namespace grpc_core
#endif
#endif
