/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/core/v3/cel.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_CORE_V3_CEL_PROTO_UPB_H__UPB_H_
#define ENVOY_CONFIG_CORE_V3_CEL_PROTO_UPB_H__UPB_H_

#include "upb/generated_code_support.h"
#include "envoy/config/core/v3/cel.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif
typedef struct envoy_config_core_v3_CelExpressionConfig {
  upb_Message UPB_PRIVATE(base);
} envoy_config_core_v3_CelExpressionConfig;




/* envoy.config.core.v3.CelExpressionConfig */
UPB_INLINE envoy_config_core_v3_CelExpressionConfig* envoy_config_core_v3_CelExpressionConfig_new(upb_Arena* arena) {
  return (envoy_config_core_v3_CelExpressionConfig*)_upb_Message_New(&envoy__config__core__v3__CelExpressionConfig_msg_init, arena);
}
UPB_INLINE envoy_config_core_v3_CelExpressionConfig* envoy_config_core_v3_CelExpressionConfig_parse(const char* buf, size_t size,
                                        upb_Arena* arena) {
  envoy_config_core_v3_CelExpressionConfig* ret = envoy_config_core_v3_CelExpressionConfig_new(arena);
  if (!ret) return NULL;
  if (upb_Decode(buf, size, UPB_UPCAST(ret), &envoy__config__core__v3__CelExpressionConfig_msg_init, NULL, 0,
                 arena) != kUpb_DecodeStatus_Ok) {
    return NULL;
  }
  return ret;
}
UPB_INLINE envoy_config_core_v3_CelExpressionConfig* envoy_config_core_v3_CelExpressionConfig_parse_ex(
    const char* buf, size_t size, const upb_ExtensionRegistry* extreg,
    int options, upb_Arena* arena) {
  envoy_config_core_v3_CelExpressionConfig* ret = envoy_config_core_v3_CelExpressionConfig_new(arena);
  if (!ret) return NULL;
  if (upb_Decode(buf, size, UPB_UPCAST(ret), &envoy__config__core__v3__CelExpressionConfig_msg_init, extreg,
                 options, arena) != kUpb_DecodeStatus_Ok) {
    return NULL;
  }
  return ret;
}
UPB_INLINE char* envoy_config_core_v3_CelExpressionConfig_serialize(const envoy_config_core_v3_CelExpressionConfig* msg,
                                      upb_Arena* arena, size_t* len) {
  char* ptr;
  (void)upb_Encode(UPB_UPCAST(msg), &envoy__config__core__v3__CelExpressionConfig_msg_init, 0, arena, &ptr, len);
  return ptr;
}
UPB_INLINE char* envoy_config_core_v3_CelExpressionConfig_serialize_ex(const envoy_config_core_v3_CelExpressionConfig* msg,
                                         int options, upb_Arena* arena,
                                         size_t* len) {
  char* ptr;
  (void)upb_Encode(UPB_UPCAST(msg), &envoy__config__core__v3__CelExpressionConfig_msg_init, options, arena, &ptr, len);
  return ptr;
}
UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_clear_enable_string_conversion(envoy_config_core_v3_CelExpressionConfig* msg) {
  const upb_MiniTableField field = {1, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_ClearBaseField(UPB_UPCAST(msg), &field);
}
UPB_INLINE bool envoy_config_core_v3_CelExpressionConfig_enable_string_conversion(const envoy_config_core_v3_CelExpressionConfig* msg) {
  bool default_val = false;
  bool ret;
  const upb_MiniTableField field = {1, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  _upb_Message_GetNonExtensionField(UPB_UPCAST(msg), &field,
                                    &default_val, &ret);
  return ret;
}
UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_clear_enable_string_concat(envoy_config_core_v3_CelExpressionConfig* msg) {
  const upb_MiniTableField field = {2, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_ClearBaseField(UPB_UPCAST(msg), &field);
}
UPB_INLINE bool envoy_config_core_v3_CelExpressionConfig_enable_string_concat(const envoy_config_core_v3_CelExpressionConfig* msg) {
  bool default_val = false;
  bool ret;
  const upb_MiniTableField field = {2, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  _upb_Message_GetNonExtensionField(UPB_UPCAST(msg), &field,
                                    &default_val, &ret);
  return ret;
}
UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_clear_enable_string_functions(envoy_config_core_v3_CelExpressionConfig* msg) {
  const upb_MiniTableField field = {3, 10, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_ClearBaseField(UPB_UPCAST(msg), &field);
}
UPB_INLINE bool envoy_config_core_v3_CelExpressionConfig_enable_string_functions(const envoy_config_core_v3_CelExpressionConfig* msg) {
  bool default_val = false;
  bool ret;
  const upb_MiniTableField field = {3, 10, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  _upb_Message_GetNonExtensionField(UPB_UPCAST(msg), &field,
                                    &default_val, &ret);
  return ret;
}

UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_set_enable_string_conversion(envoy_config_core_v3_CelExpressionConfig* msg, bool value) {
  const upb_MiniTableField field = {1, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_SetBaseField((upb_Message*)msg, &field, &value);
}
UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_set_enable_string_concat(envoy_config_core_v3_CelExpressionConfig* msg, bool value) {
  const upb_MiniTableField field = {2, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_SetBaseField((upb_Message*)msg, &field, &value);
}
UPB_INLINE void envoy_config_core_v3_CelExpressionConfig_set_enable_string_functions(envoy_config_core_v3_CelExpressionConfig* msg, bool value) {
  const upb_MiniTableField field = {3, 10, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)};
  upb_Message_SetBaseField((upb_Message*)msg, &field, &value);
}

#ifdef __cplusplus
              } /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif /* ENVOY_CONFIG_CORE_V3_CEL_PROTO_UPB_H__UPB_H_ */
