/* This file was generated by upb_generator from the input file:
 *
 *     envoy/extensions/grpc_service/call_credentials/access_token/v3/access_token_credentials.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_EXTENSIONS_GRPC_SERVICE_CALL_CREDENTIALS_ACCESS_TOKEN_V3_ACCESS_TOKEN_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_EXTENSIONS_GRPC_SERVICE_CALL_CREDENTIALS_ACCESS_TOKEN_V3_ACCESS_TOKEN_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__extensions__grpc_0service__call_0credentials__access_0token__v3__AccessTokenCredentials_msg_init;

extern const upb_MiniTableFile envoy_extensions_grpc_service_call_credentials_access_token_v3_access_token_credentials_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_EXTENSIONS_GRPC_SERVICE_CALL_CREDENTIALS_ACCESS_TOKEN_V3_ACCESS_TOKEN_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_ */
