/* This file was generated by upb_generator from the input file:
 *
 *     cel/expr/checked.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef CEL_EXPR_CHECKED_PROTO_UPB_H__UPB_MINITABLE_H_
#define CEL_EXPR_CHECKED_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable cel__expr__CheckedExpr_msg_init;
extern const upb_MiniTable cel__expr__CheckedExpr__ReferenceMapEntry_msg_init;
extern const upb_MiniTable cel__expr__CheckedExpr__TypeMapEntry_msg_init;
extern const upb_MiniTable cel__expr__Type_msg_init;
extern const upb_MiniTable cel__expr__Type__ListType_msg_init;
extern const upb_MiniTable cel__expr__Type__MapType_msg_init;
extern const upb_MiniTable cel__expr__Type__FunctionType_msg_init;
extern const upb_MiniTable cel__expr__Type__AbstractType_msg_init;
extern const upb_MiniTable cel__expr__Decl_msg_init;
extern const upb_MiniTable cel__expr__Decl__IdentDecl_msg_init;
extern const upb_MiniTable cel__expr__Decl__FunctionDecl_msg_init;
extern const upb_MiniTable cel__expr__Decl__FunctionDecl__Overload_msg_init;
extern const upb_MiniTable cel__expr__Reference_msg_init;

extern const upb_MiniTableFile cel_expr_checked_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* CEL_EXPR_CHECKED_PROTO_UPB_H__UPB_MINITABLE_H_ */
