// Copyright 2015 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef GRPC_SRC_CORE_LIB_DEBUG_TRACE_H
#define GRPC_SRC_CORE_LIB_DEBUG_TRACE_H

#include "src/core/lib/debug/trace_flags.h"  // IWYU pragma: export
#include "src/core/lib/debug/trace_impl.h"   // IWYU pragma: export

#ifdef __cplusplus
extern "C" {
#endif

int grpc_tracer_set_enabled(const char* name, int enabled);

#ifdef __cplusplus
}
#endif

#endif  // GRPC_SRC_CORE_LIB_DEBUG_TRACE_H
